@@ -163,6 +163,9 @@ class OptimizationEngine(object):
         lp_file_name_1 = os.path.join(datasets_dir, model_name + '.lp')
         model.export_as_lp(lp_file_name_1)  # Writes the .lp file
 
+
+        if ScenarioManager.env_is_cpd40():
+            ScenarioManager.add_data_file_using_ws_lib_s(lp_file_name_1)
         if ScenarioManager.env_is_cpd25():
             ScenarioManager.add_data_file_to_project_s(lp_file_name_1, model_name + '.lp')
         # Copy to csv (Not supported in CPD25. Not necessary.):
